varying mediump vec4 v_texCoord; 	// xy coords are color texture coord, zw coord are world scale transforms
varying mediump vec3 v_texOrigin;  	// xy are bump pattern tile origin, z is random start for tiles
uniform lowp vec4 u_color4;
uniform lowp vec4 u_materialAmbient;
uniform lowp vec4 u_materialDiffuse;
uniform sampler2D u_texture;
 
// A blend overlay technique. It applies color Multiply when range is dark shadows and Inverse of multiply when in highlights.
#define Blend(base, color, funcf)       vec3(funcf(base, color.r), funcf(base, color.g), funcf(base, color.b))
#define Multiply(base, color)      		(2.0 * base * color)
#define InvMultiply(base, color)      	(1.0 - 2.0 * (1.0 - base) * (1.0 - color))
#define BlendOverlay(base, color)       base <= 0.495 ? Blend(base, color, Multiply) : Blend(base, color, InvMultiply)

#define MaxRGB(channel1, channel2, channel3) 	max( max( channel1, channel2 ), channel3 )
				
void main()
{

    // Compute the bump tile coordinate to use, reverting the scale stretch applied. 
    // Span it horizontally to the bump pattern row and Repeat when off coords
    mediump vec2 bumpTiledUV = vec2(
        mod((v_texCoord.x - v_texOrigin.x) * v_texCoord.z + v_texOrigin.y + (v_texOrigin.x * 0.25), 1.0),
        mod((v_texCoord.y - v_texOrigin.y) * v_texCoord.w, 0.25) + v_texOrigin.z
    );

    lowp vec4 bumpColor = texture2D(u_texture, bumpTiledUV);

    // Get the RGB channels components of the texel and color tint them using the maxRGB technique
    lowp vec4 colorTex = texture2D(u_texture, v_texCoord.xy);
 
	// The bump pattern is overlayed on the color image
    gl_FragColor.rgb = BlendOverlay(
		bumpColor.a,
		MaxRGB(
			colorTex.r * u_materialAmbient.rgb 	/*bottom*/,
			colorTex.g * u_materialDiffuse.rgb 	/*middle*/,
			colorTex.b * u_color4.rgb 			/* top  */
		)
	);
	
    gl_FragColor.a = 1.0;
}